/******************************************************************************\
|*                                                                            *|
|*                           FaceCell Extractor 1.2                           *|
|*                                                                            *|
|* FccExtractor.h                                                             *|
|* Header file for FccExtractor module                                        *|
|*                                                                            *|
|* Copyright (C) 2006-2010 Neurotechnology                                    *|
|*                                                                            *|
\******************************************************************************/

#ifndef FCC_EXTRACTOR_H_INCLUDED
#define FCC_EXTRACTOR_H_INCLUDED

#include <NCore.h>
#include <NImages.h>
#include <NGeometry.h>

#include "FccExtractorParams.h"

#ifdef N_CPP
extern "C"
{
#endif

/// Handle to the extractor object.
N_DECLARE_HANDLE(HFccExtractor);

#ifdef N_DOCUMENTATION

/**
 * \brief  Gets information about the library.
 * \param[out] *pValue  Pointer to #NLibraryInfo that receives information
 *                      about the library.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p pValue is #NULL.</td></tr>
 *          </table>
 * \sa NLibraryInfo
 *
 * Gets information about the library as title, product name, company,
 * copyright message, and version.
 */
NResult N_API FcceGetInfo(NLibraryInfo *pValue);

#else

#ifndef N_NO_ANSI_FUNC
NResult N_API FcceGetInfoA(NLibraryInfoA *pValue);
#endif
#ifndef N_NO_UNICODE
NResult N_API FcceGetInfoW(NLibraryInfoW *pValue);
#endif
#define FcceGetInfo N_FUNC_AW(FcceGetInfo)

#endif

/**
 * \brief  Checks whether FccExtractor library is registered.
 * \return  #NTrue if library is registered, #NFalse otherwise.
 */
NBool N_API FcceIsRegistered(void);

/**
 * \brief  Creates extractor object.
 * \param[out] pHExtractor  Pointer to #HFccExtractor that receives handle to
 *                          the created extractor object.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p pHExtractor is #NULL.</td></tr>
 *            <tr><td>#N_E_OUT_OF_MEMORY</td>
 *                <td>There was not enough memory.</td></tr>
 *          </table>
 * \remark  Created object must be deleted using #FcceFree function.
 * \sa  HFccExtractor | FcceFree
 *
 * Creates extractor object and sets all parameters to default values.
 */
NResult N_API FcceCreate(HFccExtractor *pHExtractor);
/**
 * \brief  Resets extractor object.
 * \param[in]  hExtractor  Handle to the extractor object.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hExtractor is #NULL.</td></tr>
 *          </table>
 * \sa  HFccExtractor
 *
 * Sets default values for all parameters of the specified extractor.
 */
NResult N_API FcceReset(HFccExtractor hExtractor);
/**
 * \brief  Deletes extractor object.
 * \param[in]  hExtractor  Handle to the extractor object.
 * \remark  If \p hExtractor is #NULL, does nothing.
 * \sa  HFccExtractor | FcceCreate
 *
 * Deletes extractor object. After the object is deleted the specified handle
 * is no longer valid.
 */
void N_API FcceFree(HFccExtractor hExtractor);

#ifdef N_DOCUMENTATION

/**
 * \brief  Retrieves value of the parameter.
 * \param[in]  hExtractor   Handle to the extractor object.
 * \param[in]  parameterId  Identifier of the parameter to retrieve.
 * \param[out] pValue       Pointer to a variable that receives parameter
 *                          value.
 * \return  If the function succeeds and \p parameterId specifies
 *          #N_TYPE_STRING type parameter and \p pValue is #NULL, the return
 *          value is length of the string (not including NULL-terminator)
 *          \p pValue should point to.
 * \return  If the function succeeds and \p pValue is not #NULL, the return
 *          value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td rowspan="2">#N_E_ARGUMENT_NULL</td>
 *                <td>\p parameterId specifies a non-static parameter and
 *                \p hExtractor is #NULL.</td></tr>
 *            <tr><td>\p parameterId specifies a non-#N_TYPE_STRING type
 *                parameter and \p pValue is #NULL.</td></tr>
 *            <tr><td>#N_E_PARAMETER</td>
 *                <td>\p parameterId is invalid.</td></tr>
 *          </table>
 * \remark  The following values can be used for \p parameterId:
 *          - #FCCEP_MIN_IOD
 *          - #FCCEP_MAX_IOD
 *          - #FCCEP_FACE_CONFIDENCE_THRESHOLD
 *          To learn the type of parameter pass a value obtained with
 *          #NParameterMakeId macro using #N_PC_TYPE_ID code, the parameter
 *          identifier via \p parameterId and pointer to #NInt that will
 *          receive one of the N_TYPE_XXX via \p pValue parameter.
 *          \p hExtractor can be #NULL in this case.
 * \sa  HFccExtractor | FcceSetParameter
 *
 * Retrieves value of the specified parameter of the specified extractor
 * object.
 */
NResult N_API FcceGetParameter(HFccExtractor hExtractor, NUInt parameterId, void * pValue);

#else

#ifndef N_NO_ANSI_FUNC
NResult N_API FcceGetParameterA(HFccExtractor hExtractor, NUInt parameterId, void * pValue);
#endif
#ifndef N_NO_UNICODE
NResult N_API FcceGetParameterW(HFccExtractor hExtractor, NUInt parameterId, void * pValue);
#endif
#define FcceGetParameter N_FUNC_AW(FcceGetParameter)

#endif

#ifdef N_DOCUMENTATION
/**
 * \brief  Sets value of the parameter.
 * \param[in]  hExtractor   Handle to the extractor object. Can be #NULL if
 *                          setting static parameter value.
 * \param[in]  parameterId  Identifier of the parameter to set.
 * \param[in]  pValue       Pointer to the parameter value to set.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td><td>\p pValue is #NULL.</td></tr>
 *            <tr><td>#N_E_ARGUMENT_OUT_OF_RANGE</td>
 *                <td>\p pValue points to a value that is out of range.</td>
 *                </tr>
 *            <tr><td>#N_E_PARAMETER</td>
 *                <td>\p parameterId is invalid.</td></tr>
 *            <tr><td>#N_E_PARAMETER_READ_ONLY</td>
 *                <td>\p parameterId specifies a read-only parameter.</td></tr>
 *          </table>
 * \remark  The following values can be used for \p parameterId:
 *          - #FCCEP_MIN_IOD
 *          - #FCCEP_MAX_IOD
 *          - #FCCEP_FACE_CONFIDENCE_THRESHOLD
 *          To learn the type of parameter pass a value obtained with
 *          #NParameterMakeId macro using #N_PC_TYPE_ID code, the parameter
 *          identifier via \p parameterId and pointer to #NInt that will
 *          receive one of the N_TYPE_XXX via \p pValue parameter.
 *          \p hCamera can be #NULL in this case.
 * \sa  HFccExtractor | FcceGetParameter
 *
 * Sets value of the specified parameter of the specified extractor object.
 */
NResult N_API FcceSetParameter(HFccExtractor hExtractor, NUInt parameterId, const void * pValue);

#else

#ifndef N_NO_ANSI_FUNC
NResult N_API FcceSetParameterA(HFccExtractor hExtractor, NUInt parameterId, const void * pValue);
#endif
#ifndef N_NO_UNICODE
NResult N_API FcceSetParameterW(HFccExtractor hExtractor, NUInt parameterId, const void * pValue);
#endif
#define FcceSetParameter N_FUNC_AW(FcceSetParameter)

#endif

/**
 * \brief  Copies parameter values from one extractor object to another.
 * \param[in]  hDstExtractor  Handle to the destination extractor object.
 * \param[in]  hSrcExtractor  Handle to the source extractor object.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hDstExtractor or \p hSrcExtractor is #NULL.</td></tr>
 *          </table>
 * \sa  HFccExtractor
 *
 * Copies parameters values from one extractor object to another.
 */
NResult N_API FcceCopyParameters(HFccExtractor hDstExtractor, HFccExtractor hSrcExtractor);

#ifndef NL_FACE_DETECTION_ONLY

/**
 * \brief  Retrieves maximal size of feature template.
 * \param[in]  hExtractor  Handle to the extractor object.
 * \param[out] pSize       Pointer to #NSizeType that receives maximal template
 *                         size.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hExtractor or \p pSize is #NULL.</td></tr>
 *          </table>
 * \sa  HFccExtractor
 *
 * Retrieves maximal size (in bytes) of feature template that can be extracted
 * by extractor object.
 */
NResult N_API FcceGetMaxTemplateSize(HFccExtractor hExtractor, NSizeType *pSize);

#endif

// structures

/**
 * \brief  Face location.
 *
 * Face location and confidence as returned by face detection functions.
 */
typedef struct FcceFace_
{
	NRect Rectangle;     ///< Face location identified by rectangle.
	NDouble Confidence;  ///< Face confidence as returned by face detector.
} FcceFace;

/**
 * \brief  Eyes location.
 *
 * Eyes locations and confidences as returned by eyes detection functions.
 */
typedef struct FcceEyes_
{
	NPoint First;              ///< First eye location (left in the image).
	NDouble FirstConfidence;   ///< First eye confidence as returned by eye detector.
	NPoint Second;             ///< Second eye location (right in the image).
	NDouble SecondConfidence;  ///< Second eye confidence as returned by eye detector.
} FcceEyes;

/**
 * \brief  Face and eyes locations.
 *
 * Face and eyes locations merged into one structure.
 */
typedef struct FcceDetectionDetails_
{
	NBool FaceAvailable;  ///< Marks whether FcceDetectionDetails::Face field is filled.
	FcceFace Face;        ///< Face location and confidence.
	NBool EyesAvailable;  ///< Marks whether FcceDetectionDetails::Eyes field is filled.
	FcceEyes Eyes;        ///< Eyes locations and confidences.
} FcceDetectionDetails;

// face detection

/**
 * \brief  Detects one face in the image.
 * \param[in]  hExtractor  Handle to the extractor object.
 * \param[in]  hImage      Handle to the source image.
 * \param[out] pDetected   Pointer to #NBool variable that is set to #NTrue if
 *                         the face is found in the image and #NFalse
 *                         otherwise.
 * \param[out] pFace       Pointer to #FcceFace structure of found face.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hExtractor, \p hImage, \p pDetected or \p pFace is
 *                #NULL.</td></tr>
 *          </table>
 * \sa  HFccExtractor
 *
 * Finds one face region that best fits extraction requirements in the given
 * image.
 */
NResult N_API FcceDetectFace(HFccExtractor hExtractor,
	HNImage hImage,
	NBool *pDetected, FcceFace *pFace);
/**
 * \brief  Detects all faces in the image.
 * \param[in]  hExtractor  Handle to the extractor object.
 * \param[in]  hImage      Handle to the source image.
 * \param[out] pFaceCount  Pointer to #NInt variable that is set to the number
 *                         of found faces in the image.
 * \param[out] pArFaces    Pointer to an array of #FcceFace structures of found
 *                         faces.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hExtractor, \p hImage, \p pFaceCount or \p pArFaces is
 *                #NULL.</td></tr>
 *          </table>
 * \remark  The buffer for \p pArFaces variable is allocated inside the function
 *          and must be deleted by the user using #NFree function from NCore
 *          library.
 * \sa  HFccExtractor
 *
 * Finds all face regions in the given image.
 */
NResult N_API FcceDetectFaces(HFccExtractor hExtractor,
	HNImage hImage,
	NInt *pFaceCount, FcceFace **pArFaces);

// eyes detection

/**
 * \brief  Detects eyes in face region.
 * \param[in]  hExtractor  Handle to the extractor object.
 * \param[in]  hImage      Handle to the source image.
 * \param[in]  pFace       Pointer to #FcceFace structure of the face that was
 *                         found in the source image.
 * \param[out] pEyes       Pointer to #FcceEyes structures of eyes pair.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hExtractor, \p hImage, \p pFace or \p pEyes is
 *                #NULL.</td></tr>
 *          </table>
 * \sa  HFccExtractor | FcceDetectFace | FcceDetectFaces
 *
 * Detects a pair of eyes in the given face region.
 */
NResult N_API FcceDetectEyes(HFccExtractor hExtractor,
	HNImage hImage, FcceFace *pFace, FcceEyes *pEyes);

#ifndef NL_FACE_DETECTION_ONLY

// feature extraction

/**
 * \brief  Extracts face features template from the image.
 * \param[in]  hExtractor  Handle to the extractor object.
 * \param[in]  hImage      Handle to the source image.
 * \param[out] pDetectionDetails  Pointer to #FcceDetectionDetails structure
 *                                of face and eyes detection results.
 * \param[out] pBuffer     Pointer to the memory space allocated for extracted
 *                         features.
 * \param[in]  bufferSize  Size of allocated buffer (must be greater or equal
 *                         to a maximum possible template size as returned by
 *                         #FcceGetMaxTemplateSize).
 * \param[out] *pSize  Returns number of bytes filled in the allocated buffer.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hExtractor, \p hImage, \p pDetectionDetails,
 *                \p pBuffer or \p pSize is #NULL.</td></tr>
 *          </table>
 * \sa  HFccExtractor
 *
 * Fully automatically extracts face features template of the face that best
 * fits extraction requirements in the given image.
 */
NResult N_API FcceExtract(HFccExtractor hExtractor,
	HNImage hImage,
	FcceDetectionDetails *pDetectionDetails,
	void *pBuffer, NSizeType bufferSize, NSizeType *pSize);
/**
 * \brief  Initializes face features template extraction from the stream.
 * \param[in]  hExtractor        Handle to the extractor object.
 * \param[in]  durationInFrames  Number of images that will be used in
 *                               extraction.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hExtractor is #NULL.</td></tr>
 *            <tr><td>#N_E_ARGUMENT_OUT_OF_RANGE</td>
 *                <td>\p durationInFrames is less than 1.</td></tr>
 *          </table>
 * \sa  HFccExtractor | FcceExtractNext
 *
 * Initializes face fetures template extraction routines that extract facial
 * features from the given sequence of images.
 */
NResult N_API FcceExtractStart(HFccExtractor hExtractor,
	NInt durationInFrames);
/**
 * \brief  Extracts face features template from the sequence of images.
 * \param[in]  hExtractor  Handle to the extractor object.
 * \param[in]  hImage      Handle to the source image.
 * \param[out] pDetectionDetails  Pointer to #FcceDetectionDetails structure
 *                                of face and eyes detection results.
 * \param[out] pBuffer     Pointer to the memory space allocated for extracted
 *                         features.
 * \param[in]  bufferSize  Size of allocated buffer (must be greater or equal
 *                         to a maximum possible template size as returned by
 *                         #FcceGetMaxTemplateSize).
 * \param[out] pSize       Returns number of bytes filled in the allocated
 *                         buffer.
 * \param[out] pStopped    Pointer to #NBool variable that is set to #NTrue if
 *                         extraction process is completed and #NFalse
 *                         otherwise.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hExtractor, \p hImage, \p pDetectionDetails,
 *                \p pBuffer, \p pSize or \p pStopped is #NULL.</td></tr>
 *          </table>
 * \remark  When extraction process is completed (\p pStopped is set to #NTrue)
 *          one of the following has happened:
 *          - all frames were processed and face features template was created
 *            successfully (\p pSize value is greater than 0).
 *          - some first frames were processed and face features templates was
 *            created successfully (\p pSize value is greater than 0).
 *          - all frames were processed and face features template was not
 *            created (\p pSize value is 0).
 * \sa  HFccExtractor | FcceExtractStart
 *
 * Fully automatically extracts face features template of the face that best
 * fits extraction requirements in the given sequence of images that must be
 * fed to this function one by one.
 */
NResult N_API FcceExtractNext(HFccExtractor hExtractor,
	HNImage hImage,
	FcceDetectionDetails *pDetectionDetails,
	void *pBuffer, NSizeType bufferSize, NSizeType *pSize, NBool *pStopped);
/**
 * \brief  Extracts face features template from the image when coordinates of
 *         eyes are known.
 * \param[in]  hExtractor  Handle to the extractor object.
 * \param[in]  hImage      Handle to the source image.
 * \param[in]  pEyes       Pointer to #FcceEyes structure of eyes pair in the
 *                         source image.
 * \param[out] pBuffer     Pointer to the memory space allocated for extracted
 *                         features.
 * \param[in]  bufferSize  Size of allocated buffer (must be greater or equal
 *                         to a maximum possible template size as returned by
 *                         #FcceGetMaxTemplateSize).
 * \param[out] pSize       Returns number of bytes filled in the allocated
 *                         buffer.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hExtractor, \p hImage, \p pEyes,
 *                \p pBuffer or \p pSize is #NULL.</td></tr>
 *          </table>
 * \remark  If eyes were annotated manually confidence for both eyes must be
 *          set to 100.
 * \sa  HFccExtractor | FcceDetectEyes
 *
 * Extracts face features template from the given image and coordinates of
 * eyes.
 */
NResult N_API FcceExtractUsingEyes(HFccExtractor hExtractor,
	HNImage hImage, FcceEyes *pEyes,
	void *pBuffer, NSizeType bufferSize, NSizeType *pSize);

// generalization

/**
 * \brief  Generates a generalized face features template.
 * \param[in]  hExtractor       Handle to the extractor object.
 * \param[in]  templateCount    Number of templates in \p arTemplates array.
 * \param[in]  arTemplates      Pointer to the array of templates.
 * \param[in]  arTemplateSizes  Pointer to the array with sizes of each
 *                              template in bytes.
 * \param[out] pBuffer          Pointer to the memory space allocated for
 *                              extracted features.
 * \param[in]  bufferSize       Size of allocated buffer (must be greater or
 *                              equal to a maximum possible template size as
 *                              returned by #FcceGetMaxTemplateSize).
 * \param[out] pSize            Returns number of bytes filled in the allocated
 *                              buffer.
 * \return  If the function succeeds the return value is #N_OK.
 * \return  If the function fails, the return value is one of the following
 *          error codes:
 *          <table>
 *            <tr><th>Error Code</th><th>Condition</th></tr>
 *            <tr><td>#N_E_ARGUMENT_NULL</td>
 *                <td>\p hExtractor, \p arTemplates, \p arTemplateSizes,
 *                \p pBuffer or \p pSize is #NULL.</td></tr>
 *            <tr><td>#N_E_ARGUMENT</td>
 *                <td>One or more templates in \p arTemplates array is
 *                compressed.</td></tr>
 *          </table>
 * \sa  HFccExtractor | FcceDetectEyes
 *
 * Generates a generalized face features template from the set of face features
 * templates.
 */
NResult N_API FcceGeneralize(HFccExtractor hExtractor,
	NInt templateCount, const void **arTemplates, const NSizeType *arTemplateSizes,
	void *pBuffer, NSizeType bufferSize, NSizeType *pSize);

#endif

#ifdef N_CPP
}
#endif

#endif // !FCC_EXTRACTOR_H_INCLUDED
