// system headers
#include <stdio.h>
#include <stdlib.h>

// global FaceCell SDK headers
#include <NCore.h>
#include <NImages.h>
#include <FccExtractor.h>

char title[] = "FaceCell Enroll from Stream Tutorial, Version 1.0.0.0, Copyright (C) 2006-2007 Neurotechnology\n";

int usage()
{
	printf("usage:\n");
	printf("\tEnrollStream [Template] [Image]...\n");
	printf("\n");
	printf("\t[Template] - filename for template.\n");
	printf("\t[Image]... - one or more filenames of images.\n");

	return 1;
}

int main(int argc, char **argv)
{
	NResult result;
	HFccExtractor extractor;

	HNImage image;
	HNImage grayscale;

	FcceDetectionDetails details;
	NSizeType maxSize, size;
	void *template;
	NBool stopped;

	int i;
	FILE *file;

	printf(title);
	if (argc < 3)
	{
		return usage();
	}

	// create an extractor
	result = FcceCreate(&extractor);
	if (NFailed(result))
	{
		printf("extractor creation failed (result = %d)!\n", result);
		return -1;
	}

	// get maximum template size
	result = FcceGetMaxTemplateSize(extractor, &maxSize);
	if (NFailed(result))
	{
		printf("could not get max template size (result = %d)!\n", result);
		FcceFree(extractor);
		return -1;
	}

	// start extraction of face template from the stream (sequence of images)
	template = (void *) malloc(maxSize);
	result = FcceExtractStart(extractor, argc - 2);
	if (NFailed(result))
	{
		printf("starting of extraction from stream failed (result = %d)!\n", result);
		FcceFree(extractor);
		free(template);
		return -1;
	}

	// process stream of images
	stopped = NFalse;
	size = 0;
	for (i = 2; !stopped && (i < argc); ++i)
	{
		// read image
		result = NImageCreateFromFile(argv[i], NULL, &image);
		if (NFailed(result))
		{
			printf("image[%d] loading failed (result = %d)!\n", i - 2, result);
			return -1;
		}

		// convert image to grayscale
		result = NImageCreateFromImage(npfGrayscale, 0, image, &grayscale);
		NImageFree(image);
		if (NFailed(result))
		{
			printf("image[%d] conversion to grayscale failed (result = %d)!\n", i - 2, result);
			return -1;
		}

		// use image as another frame of stream
		result = FcceExtractNext(extractor, grayscale, &details, template, maxSize, &size, &stopped);
		NImageFree(grayscale);
		if (NFailed(result))
		{
			printf("template[%d] extraction failed (result = %d)!\n", i - 2, result);
			FcceFree(extractor);
			free(template);
			return -1;
		}
		if (details.FaceAvailable)
		{
			printf("found face:\n");
			printf("\tlocation = (%d, %d), width = %d, height = %d, confidence = %.2f\n",
				details.Face.Rectangle.X, details.Face.Rectangle.Y,
				details.Face.Rectangle.Width, details.Face.Rectangle.Height,
				details.Face.Confidence);
		}
		if (details.EyesAvailable)
		{
			printf("\tfound eyes:\n");
			printf("\t\tfirst: location = (%d, %d), confidence = %.2f\n",
				details.Eyes.First.X, details.Eyes.First.Y,
				details.Eyes.FirstConfidence);
			printf("\t\tsecond: location = (%d, %d), confidence = %.2f\n",
				details.Eyes.Second.X, details.Eyes.Second.Y,
				details.Eyes.SecondConfidence);
		}
	}
	if (size == 0)
	{
		printf("template creation failed (size = %d)!\n", size);
		FcceFree(extractor);
		free(template);
		return -1;
	}
	else
	{
		printf("extracted %d bytes to template.\n", size);
	}

	// save template to file
	file = fopen(argv[1], "wb");
	if (file == NULL)
	{
		printf("could not open file \"%s\" for writing!\n", argv[1]);
	}
	else
	{
		fwrite(template, 1, size, file);
		fclose(file);
	}

	// free resources
	FcceFree(extractor);
	free(template);

	return 0;
}
