// system headers
#include <stdio.h>
#include <stdlib.h>

// global FaceCell SDK headers
#include <NCore.h>
#include <NImages.h>
#include <FccExtractor.h>
#include <FccMatcher.h>

char title[] = "FaceCell Verify Tutorial, Version 1.0.0.0, Copyright (C) 2006-2007 Neurotechnology\n";

int usage()
{
	printf("usage:\n");
	printf("\tVerify [Image] [Template]\n");
	printf("\n");
	printf("\t[Image] - filename of image.\n");
	printf("\t[Template] - filename of templates.\n");

	return 1;
}

int main(int argc, char **argv)
{
	NResult result;
	HFccExtractor extractor;
	HFccMatcher matcher;

	HNImage image, grayscale;

	NSizeType enrolledSize;
	void *enrolledTemplate;
	FcceDetectionDetails details;
	NSizeType maxSize, size;
	void *template;
	NDouble score;

	FILE *file;

	printf(title);
	if (argc < 3)
	{
		return usage();
	}

	// read image
	result = NImageCreateFromFile(argv[1], NULL, &image);
	if (NFailed(result))
	{
		printf("image loading failed (result = %d)!\n", result);
		return -1;
	}

	// convert image to grayscale
	result = NImageCreateFromImage(npfGrayscale, 0, image, &grayscale);
	NImageFree(image);
	if (NFailed(result))
	{
		printf("image conversion to grayscale failed (result = %d)!\n", result);
		return -1;
	}

	// read enrolled templates
	file = fopen(argv[2], "rb");
	if (file == NULL)
	{
		printf("cannot read template file!\n");
		NImageFree(grayscale);
		return -1;
	}

	fseek(file, 0, SEEK_END);
	enrolledSize = (NSizeType)ftell(file);
	fseek(file, 0, SEEK_SET);

	enrolledTemplate = (void *) malloc(enrolledSize);
	fread(enrolledTemplate, 1, enrolledSize, file);
	fclose(file);

	// create an extractor
	result = FcceCreate(&extractor);
	if (NFailed(result))
	{
		printf("extractor creation failed (result = %d)!\n", result);
		NImageFree(grayscale);
		free(enrolledTemplate);
		return -1;
	}

	// get maximum template size
	result = FcceGetMaxTemplateSize(extractor, &maxSize);
	if (NFailed(result))
	{
		printf("could not get max template size (result = %d)!\n", result);
		NImageFree(grayscale);
		free(enrolledTemplate);
		FcceFree(extractor);
		return -1;
	}

	// extract a face template from the image
	template = (void *) malloc(maxSize);
	result = FcceExtract(extractor, grayscale, &details, template, maxSize, &size);
	if (NFailed(result) || (size == 0))
	{
		printf("template extraction failed (result = %d, template size = %d)!\n",
			result, size);
		NImageFree(grayscale);
		free(enrolledTemplate);
		FcceFree(extractor);
		free(template);
		return -1;
	}
	if (details.FaceAvailable)
	{
		printf("found face:\n");
		printf("\tlocation = (%d, %d), width = %d, height = %d, confidence = %.2f\n",
			details.Face.Rectangle.X, details.Face.Rectangle.Y,
			details.Face.Rectangle.Width, details.Face.Rectangle.Height,
			details.Face.Confidence);
	}
	if (details.EyesAvailable)
	{
		printf("\tfound eyes:\n");
		printf("\t\tfirst: location = (%d, %d), confidence = %.2f\n",
			details.Eyes.First.X, details.Eyes.First.Y,
			details.Eyes.FirstConfidence);
		printf("\t\tsecond: location = (%d, %d), confidence = %.2f\n",
			details.Eyes.Second.X, details.Eyes.Second.Y,
			details.Eyes.SecondConfidence);
	}
	printf("extracted %d bytes to template.\n", size);
	NImageFree(grayscale);
	FcceFree(extractor);

	// create a matcher
	result = FccmCreate(&matcher);
	if (NFailed(result))
	{
		printf("matcher creation failed (result = %d)!\n", result);
		free(enrolledTemplate);
		free(template);
		return -1;
	}

	// perform verification
	result = FccmVerify(matcher, template, size, enrolledTemplate, enrolledSize, &score);
	if (NFailed(result))
	{
		printf("verification failed (result = %d)!\n", result);
		free(enrolledTemplate);
		free(template);
		FccmFree(matcher);
		return -1;
	}
	printf("image scored %.4f, verification.. ", score);
	if (score > 0.0)
	{
		printf("succedded.\n");
	}
	else
	{
		printf("failed.\n");
	}

	// free resources
	free(enrolledTemplate);
	free(template);
	FccmFree(matcher);

	return 0;
}
